"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.watchFiles = void 0;
const typescript = require("./typescript");
const fs_1 = require("fs");
const globby_1 = require("globby");
const path = require('path');
const shell = require('shelljs');
const determineNewWatchFiles = (watchedFiles, newWatchFiles, cb) => {
    watchedFiles.forEach((fileName) => {
        if (newWatchFiles.indexOf(fileName) < 0) {
            fs_1.unwatchFile(fileName, cb);
        }
    });
    newWatchFiles.forEach((fileName) => {
        if (watchedFiles.indexOf(fileName) < 0) {
            fs_1.watchFile(fileName, { persistent: true, interval: 250 }, cb);
        }
    });
};
function watchFiles(rootFileNames, originalServicePath, cb, generateGraphqlTypes, graphqlFilePaths) {
    return __awaiter(this, void 0, void 0, function* () {
        const tsConfig = typescript.getTypescriptConfig(originalServicePath);
        let watchedFiles = typescript.getSourceFiles(rootFileNames, tsConfig);
        let watchedGraphqlFiles = yield globby_1.default(graphqlFilePaths);
        const watchCallback = (curr, prev) => {
            // Check timestamp
            if (+curr.mtime <= +prev.mtime) {
                return;
            }
            cb();
            // use can reference not watched yet file or remove reference to already watched
            const newWatchFiles = typescript.getSourceFiles(rootFileNames, tsConfig);
            determineNewWatchFiles(watchedFiles, newWatchFiles, generateGraphqlTypes);
            watchedFiles = newWatchFiles;
        };
        const watchGraphqlCallback = (changedFilePath) => (curr, prev) => __awaiter(this, void 0, void 0, function* () {
            // Check timestamp
            if (+curr.mtime <= +prev.mtime) {
                return;
            }
            const copyChangedGraphqlFileToBuild = () => {
                const basePath = path.basename(path.resolve(process.cwd()));
                const basePathIndex = changedFilePath.indexOf(basePath) + basePath.length;
                const buildPath = changedFilePath.slice(0, basePathIndex) +
                    '/.build' +
                    changedFilePath.slice(basePathIndex);
                shell.cp(changedFilePath, buildPath);
            };
            generateGraphqlTypes();
            copyChangedGraphqlFileToBuild();
            // use can reference not watched yet file or remove reference to already watched
            const newWatchGqlFiles = yield globby_1.default(graphqlFilePaths);
            determineNewWatchFiles(watchedGraphqlFiles, newWatchGqlFiles, generateGraphqlTypes);
            watchedGraphqlFiles = newWatchGqlFiles;
        });
        watchedFiles.forEach((fileName) => {
            fs_1.watchFile(fileName, { persistent: true, interval: 250 }, watchCallback);
        });
        if (watchedGraphqlFiles.length) {
            watchedGraphqlFiles.forEach((fileName) => {
                fs_1.watchFile(fileName, { persistent: true, interval: 250 }, watchGraphqlCallback(fileName));
            });
        }
    });
}
exports.watchFiles = watchFiles;
//# sourceMappingURL=watchFiles.js.map